# This file was auto-generated by Fern from our API Definition.

from .types import EntityUserId, EntityUserRequest, EntityUserResponse

__all__ = ["EntityUserId", "EntityUserRequest", "EntityUserResponse"]



# This file was auto-generated by Fern from our API Definition.

from .entity_user_id import EntityUserId
from .entity_user_request import EntityUserRequest
from .entity_user_response import EntityUserResponse

__all__ = ["EntityUserId", "EntityUserRequest", "EntityUserResponse"]



# This file was auto-generated by Fern from our API Definition.

from .counterparty_response import CounterpartyResponse
from .find_counterparties_response import FindCounterpartiesResponse

__all__ = ["CounterpartyResponse", "FindCounterpartiesResponse"]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    CommentId,
    CommentRequest,
    CommentResponse,
    CreateVendorRequest,
    CurrencyCode,
    DocumentResponse,
    InvoiceId,
    InvoiceLineItemRequest,
    InvoiceLineItemResponse,
    InvoiceRequest,
    InvoiceResponse,
    InvoiceStatus,
)

__all__ = [
    "CommentId",
    "CommentRequest",
    "CommentResponse",
    "CreateVendorRequest",
    "CurrencyCode",
    "DocumentResponse",
    "InvoiceId",
    "InvoiceLineItemRequest",
    "InvoiceLineItemResponse",
    "InvoiceRequest",
    "InvoiceResponse",
    "InvoiceStatus",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ...entity.types.entity_id import EntityId
from ...entity_users.types.entity_user_id import EntityUserId
from ...payment_method.types.payment_method_id import PaymentMethodId
from .create_vendor_request import CreateVendorRequest
from .currency_code import CurrencyCode
from .invoice_line_item_request import InvoiceLineItemRequest
from .invoice_status import InvoiceStatus


class InvoiceRequest(pydantic.BaseModel):
    status: typing.Optional[InvoiceStatus]
    amount: typing.Optional[float]
    currency: typing.Optional[CurrencyCode]
    invoice_date: typing.Optional[dt.datetime] = pydantic.Field(
        alias="invoiceDate", description=("Date the invoice was created.\n")
    )
    deduction_date: typing.Optional[dt.datetime] = pydantic.Field(
        alias="deductionDate", description=("Date when funds will be deducted from payer's account.\n")
    )
    funded_date: typing.Optional[dt.datetime] = pydantic.Field(
        alias="fundedDate", description=("Date of funds settlement.\n")
    )
    due_date: typing.Optional[dt.datetime] = pydantic.Field(alias="dueDate", description=("Due date of invoice.\n"))
    invoice_number: typing.Optional[str] = pydantic.Field(alias="invoiceNumber")
    note_to_self: typing.Optional[str] = pydantic.Field(alias="noteToSelf")
    service_start_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceStartDate")
    service_end_date: typing.Optional[dt.datetime] = pydantic.Field(alias="serviceEndDate")
    payer_id: typing.Optional[EntityId] = pydantic.Field(alias="payerId")
    payment_source_id: typing.Optional[PaymentMethodId] = pydantic.Field(alias="paymentSourceId")
    vendor_id: typing.Optional[EntityId] = pydantic.Field(alias="vendorId")
    payment_destination_id: typing.Optional[PaymentMethodId] = pydantic.Field(alias="paymentDestinationId")
    create_vendor: typing.Optional[CreateVendorRequest] = pydantic.Field(
        alias="createVendor",
        description=(
            "When paying to a new vendor, use the createVendor object. Mercoa will create the vendor entity and tie it to this invoice. This object is ignored when updating an invoice.\n"
        ),
    )
    update_vendor: typing.Optional[CreateVendorRequest] = pydantic.Field(
        alias="updateVendor",
        description=(
            "When paying to an existing vendor with an incomplete profile, use the updateVendor object. Mercoa will update the vendor entity tied to this invoice. This object is ignored if the vendor already has already been created with complete information and when creating a new invoice.\n"
        ),
    )
    line_items: typing.Optional[typing.List[InvoiceLineItemRequest]] = pydantic.Field(alias="lineItems")
    uploaded_image: typing.Optional[str] = pydantic.Field(
        alias="uploadedImage",
        description=("Base64 encoded image or PDF of invoice. PNG, JPG, and PDF are supported. 10MB max.\n"),
    )
    created_by_id: typing.Optional[EntityUserId] = pydantic.Field(
        alias="createdById", description=("ID of entity user who created this invoice.\n")
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .account_type import AccountType
from .business_profile_request import BusinessProfileRequest
from .business_profile_response import BusinessProfileResponse
from .business_type import BusinessType
from .ein import Ein
from .entity_add_payees_request import EntityAddPayeesRequest
from .entity_id import EntityId
from .entity_request import EntityRequest
from .entity_response import EntityResponse
from .entity_status import EntityStatus
from .entity_update_request import EntityUpdateRequest
from .individual_profile_request import IndividualProfileRequest
from .individual_profile_response import IndividualProfileResponse
from .invoice_metrics_response import InvoiceMetricsResponse
from .profile_request import ProfileRequest
from .profile_response import ProfileResponse
from .tax_id import TaxID

__all__ = [
    "AccountType",
    "BusinessProfileRequest",
    "BusinessProfileResponse",
    "BusinessType",
    "Ein",
    "EntityAddPayeesRequest",
    "EntityId",
    "EntityRequest",
    "EntityResponse",
    "EntityStatus",
    "EntityUpdateRequest",
    "IndividualProfileRequest",
    "IndividualProfileResponse",
    "InvoiceMetricsResponse",
    "ProfileRequest",
    "ProfileResponse",
    "TaxID",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    BankAccountId,
    BankAccountRequest,
    BankAccountResponse,
    BankStatus,
    BankType,
    CardBrand,
    CardId,
    CardRequest,
    CardResponse,
    CardType,
    CheckId,
    CheckRequest,
    CheckResponse,
    CustomId,
    CustomPaymentMethodRequest,
    CustomPaymentMethodResponse,
    PaymentMethodId,
    PaymentMethodRequest,
    PaymentMethodResponse,
    PaymentMethodType,
)

__all__ = [
    "BankAccountId",
    "BankAccountRequest",
    "BankAccountResponse",
    "BankStatus",
    "BankType",
    "CardBrand",
    "CardId",
    "CardRequest",
    "CardResponse",
    "CardType",
    "CheckId",
    "CheckRequest",
    "CheckResponse",
    "CustomId",
    "CustomPaymentMethodRequest",
    "CustomPaymentMethodResponse",
    "PaymentMethodId",
    "PaymentMethodRequest",
    "PaymentMethodResponse",
    "PaymentMethodType",
]



# This file was auto-generated by Fern from our API Definition.

from .bank_account_id import BankAccountId
from .bank_account_request import BankAccountRequest
from .bank_account_response import BankAccountResponse
from .bank_status import BankStatus
from .bank_type import BankType
from .card_brand import CardBrand
from .card_id import CardId
from .card_request import CardRequest
from .card_response import CardResponse
from .card_type import CardType
from .check_id import CheckId
from .check_request import CheckRequest
from .check_response import CheckResponse
from .custom_id import CustomId
from .custom_payment_method_request import CustomPaymentMethodRequest
from .custom_payment_method_response import CustomPaymentMethodResponse
from .payment_method_id import PaymentMethodId
from .payment_method_request import PaymentMethodRequest
from .payment_method_response import PaymentMethodResponse
from .payment_method_type import PaymentMethodType

__all__ = [
    "BankAccountId",
    "BankAccountRequest",
    "BankAccountResponse",
    "BankStatus",
    "BankType",
    "CardBrand",
    "CardId",
    "CardRequest",
    "CardResponse",
    "CardType",
    "CheckId",
    "CheckRequest",
    "CheckResponse",
    "CustomId",
    "CustomPaymentMethodRequest",
    "CustomPaymentMethodResponse",
    "PaymentMethodId",
    "PaymentMethodRequest",
    "PaymentMethodResponse",
    "PaymentMethodType",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from .bank_account_id import BankAccountId
from .bank_type import BankType


class BankAccountRequest(pydantic.BaseModel):
    id: typing.Optional[BankAccountId]
    bank_name: str = pydantic.Field(alias="bankName")
    routing_number: str = pydantic.Field(alias="routingNumber")
    account_number: str = pydantic.Field(alias="accountNumber")
    plaid_public_token: typing.Optional[str] = pydantic.Field(
        alias="plaidPublicToken", description=("Public token from Plaid Link\n")
    )
    account_type: BankType = pydantic.Field(alias="accountType")

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import RepresentativeId, RepresentativeRequest, RepresentativeResponse, Responsibilities

__all__ = ["RepresentativeId", "RepresentativeRequest", "RepresentativeResponse", "Responsibilities"]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from ...core.api_error import ApiError
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_headers import remove_none_from_headers
from ...environment import MercoaEnvironment
from ..organization.types.organization_id import OrganizationId
from .types.email_ocr_request import EmailOcrRequest
from .types.ocr_response import OCRResponse


class OcrClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    def ocr(self, *, mime_type: str, image: str) -> OCRResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "ocr"),
            json=jsonable_encoder({"mimeType": mime_type, "image": image}),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OCRResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def email_inbox(self, *, org: OrganizationId, items: typing.List[EmailOcrRequest]) -> None:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "emailOcr"),
            params={"org": org},
            json=jsonable_encoder({"items": items}),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncOcrClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    async def ocr(self, *, mime_type: str, image: str) -> OCRResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "ocr"),
                json=jsonable_encoder({"mimeType": mime_type, "image": image}),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(OCRResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def email_inbox(self, *, org: OrganizationId, items: typing.List[EmailOcrRequest]) -> None:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "emailOcr"),
                params={"org": org},
                json=jsonable_encoder({"items": items}),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from backports.cached_property import cached_property

from .environment import MercoaEnvironment
from .resources.bank_lookup.client import AsyncBankLookupClient, BankLookupClient
from .resources.counterparty.client import AsyncCounterpartyClient, CounterpartyClient
from .resources.entity.client import AsyncEntityClient, EntityClient
from .resources.entity_users.client import AsyncEntityUsersClient, EntityUsersClient
from .resources.invoice.client import AsyncInvoiceClient, InvoiceClient
from .resources.ocr.client import AsyncOcrClient, OcrClient
from .resources.organization.client import AsyncOrganizationClient, OrganizationClient
from .resources.payment_method.client import AsyncPaymentMethodClient, PaymentMethodClient
from .resources.payment_method_schema.client import AsyncPaymentMethodSchemaClient, PaymentMethodSchemaClient
from .resources.process_invoice.client import AsyncProcessInvoiceClient, ProcessInvoiceClient
from .resources.representative.client import AsyncRepresentativeClient, RepresentativeClient
from .resources.transaction.client import AsyncTransactionClient, TransactionClient


class Mercoa:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    @cached_property
    def bank_lookup(self) -> BankLookupClient:
        return BankLookupClient(environment=self._environment, token=self._token)

    @cached_property
    def counterparty(self) -> CounterpartyClient:
        return CounterpartyClient(environment=self._environment, token=self._token)

    @cached_property
    def entity_users(self) -> EntityUsersClient:
        return EntityUsersClient(environment=self._environment, token=self._token)

    @cached_property
    def entity(self) -> EntityClient:
        return EntityClient(environment=self._environment, token=self._token)

    @cached_property
    def invoice(self) -> InvoiceClient:
        return InvoiceClient(environment=self._environment, token=self._token)

    @cached_property
    def ocr(self) -> OcrClient:
        return OcrClient(environment=self._environment, token=self._token)

    @cached_property
    def organization(self) -> OrganizationClient:
        return OrganizationClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method_schema(self) -> PaymentMethodSchemaClient:
        return PaymentMethodSchemaClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method(self) -> PaymentMethodClient:
        return PaymentMethodClient(environment=self._environment, token=self._token)

    @cached_property
    def process_invoice(self) -> ProcessInvoiceClient:
        return ProcessInvoiceClient(environment=self._environment, token=self._token)

    @cached_property
    def representative(self) -> RepresentativeClient:
        return RepresentativeClient(environment=self._environment, token=self._token)

    @cached_property
    def transaction(self) -> TransactionClient:
        return TransactionClient(environment=self._environment, token=self._token)


class AsyncMercoa:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    @cached_property
    def bank_lookup(self) -> AsyncBankLookupClient:
        return AsyncBankLookupClient(environment=self._environment, token=self._token)

    @cached_property
    def counterparty(self) -> AsyncCounterpartyClient:
        return AsyncCounterpartyClient(environment=self._environment, token=self._token)

    @cached_property
    def entity_users(self) -> AsyncEntityUsersClient:
        return AsyncEntityUsersClient(environment=self._environment, token=self._token)

    @cached_property
    def entity(self) -> AsyncEntityClient:
        return AsyncEntityClient(environment=self._environment, token=self._token)

    @cached_property
    def invoice(self) -> AsyncInvoiceClient:
        return AsyncInvoiceClient(environment=self._environment, token=self._token)

    @cached_property
    def ocr(self) -> AsyncOcrClient:
        return AsyncOcrClient(environment=self._environment, token=self._token)

    @cached_property
    def organization(self) -> AsyncOrganizationClient:
        return AsyncOrganizationClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method_schema(self) -> AsyncPaymentMethodSchemaClient:
        return AsyncPaymentMethodSchemaClient(environment=self._environment, token=self._token)

    @cached_property
    def payment_method(self) -> AsyncPaymentMethodClient:
        return AsyncPaymentMethodClient(environment=self._environment, token=self._token)

    @cached_property
    def process_invoice(self) -> AsyncProcessInvoiceClient:
        return AsyncProcessInvoiceClient(environment=self._environment, token=self._token)

    @cached_property
    def representative(self) -> AsyncRepresentativeClient:
        return AsyncRepresentativeClient(environment=self._environment, token=self._token)

    @cached_property
    def transaction(self) -> AsyncTransactionClient:
        return AsyncTransactionClient(environment=self._environment, token=self._token)



# This file was auto-generated by Fern from our API Definition.

from . import (
    bank_lookup,
    commons,
    counterparty,
    entity,
    entity_users,
    invoice,
    ocr,
    organization,
    payment_method,
    payment_method_schema,
    process_invoice,
    representative,
    transaction,
)
from .bank_lookup import BankAddress, BankLookupResponse
from .commons import ITIN, SSN, Address, BirthDate, FullName, IndividualGovernmentID, PhoneNumber, UnauthorizedError
from .counterparty import CounterpartyResponse, FindCounterpartiesResponse
from .entity import (
    AccountType,
    BusinessProfileRequest,
    BusinessProfileResponse,
    BusinessType,
    Ein,
    EntityAddPayeesRequest,
    EntityId,
    EntityRequest,
    EntityResponse,
    EntityStatus,
    EntityUpdateRequest,
    IndividualProfileRequest,
    IndividualProfileResponse,
    InvoiceMetricsResponse,
    ProfileRequest,
    ProfileResponse,
    TaxID,
)
from .entity_users import EntityUserId, EntityUserRequest, EntityUserResponse
from .invoice import (
    CommentId,
    CommentRequest,
    CommentResponse,
    CreateVendorRequest,
    CurrencyCode,
    DocumentResponse,
    InvoiceId,
    InvoiceLineItemRequest,
    InvoiceLineItemResponse,
    InvoiceRequest,
    InvoiceResponse,
    InvoiceStatus,
)
from .ocr import Attachments, EmailOcrRequest, OcrMailbox, OCRResponse
from .organization import (
    ColorSchemeRequest,
    ColorSchemeResponse,
    EmailLogResponse,
    EmailProviderRequest,
    EmailProviderResponse,
    EmailSenderProvider,
    EmailSenderRequest,
    EmailSenderResponse,
    OrganizationId,
    OrganizationRequest,
    OrganizationResponse,
    PaymentMethodsRequest,
    PaymentMethodsResponse,
    PaymentRailMarkup,
    PaymentRailMarkupType,
    PaymentRailRequest,
    PaymentRailResponse,
)
from .payment_method import (
    BankAccountId,
    BankAccountRequest,
    BankAccountResponse,
    BankStatus,
    BankType,
    CardBrand,
    CardId,
    CardRequest,
    CardResponse,
    CardType,
    CheckId,
    CheckRequest,
    CheckResponse,
    CustomId,
    CustomPaymentMethodRequest,
    CustomPaymentMethodResponse,
    PaymentMethodId,
    PaymentMethodRequest,
    PaymentMethodResponse,
    PaymentMethodType,
)
from .payment_method_schema import (
    PaymentMethodSchemaField,
    PaymentMethodSchemaFieldType,
    PaymentMethodSchemaId,
    PaymentMethodSchemaRequest,
    PaymentMethodSchemaResponse,
)
from .process_invoice import ProcessInvoiceRequest
from .representative import RepresentativeId, RepresentativeRequest, RepresentativeResponse, Responsibilities
from .transaction import TransactionId, TransactionResponse, TransactionResponseExpanded, TransactionStatus

__all__ = [
    "AccountType",
    "Address",
    "Attachments",
    "BankAccountId",
    "BankAccountRequest",
    "BankAccountResponse",
    "BankAddress",
    "BankLookupResponse",
    "BankStatus",
    "BankType",
    "BirthDate",
    "BusinessProfileRequest",
    "BusinessProfileResponse",
    "BusinessType",
    "CardBrand",
    "CardId",
    "CardRequest",
    "CardResponse",
    "CardType",
    "CheckId",
    "CheckRequest",
    "CheckResponse",
    "ColorSchemeRequest",
    "ColorSchemeResponse",
    "CommentId",
    "CommentRequest",
    "CommentResponse",
    "CounterpartyResponse",
    "CreateVendorRequest",
    "CurrencyCode",
    "CustomId",
    "CustomPaymentMethodRequest",
    "CustomPaymentMethodResponse",
    "DocumentResponse",
    "Ein",
    "EmailLogResponse",
    "EmailOcrRequest",
    "EmailProviderRequest",
    "EmailProviderResponse",
    "EmailSenderProvider",
    "EmailSenderRequest",
    "EmailSenderResponse",
    "EntityAddPayeesRequest",
    "EntityId",
    "EntityRequest",
    "EntityResponse",
    "EntityStatus",
    "EntityUpdateRequest",
    "EntityUserId",
    "EntityUserRequest",
    "EntityUserResponse",
    "FindCounterpartiesResponse",
    "FullName",
    "ITIN",
    "IndividualGovernmentID",
    "IndividualProfileRequest",
    "IndividualProfileResponse",
    "InvoiceId",
    "InvoiceLineItemRequest",
    "InvoiceLineItemResponse",
    "InvoiceMetricsResponse",
    "InvoiceRequest",
    "InvoiceResponse",
    "InvoiceStatus",
    "OCRResponse",
    "OcrMailbox",
    "OrganizationId",
    "OrganizationRequest",
    "OrganizationResponse",
    "PaymentMethodId",
    "PaymentMethodRequest",
    "PaymentMethodResponse",
    "PaymentMethodSchemaField",
    "PaymentMethodSchemaFieldType",
    "PaymentMethodSchemaId",
    "PaymentMethodSchemaRequest",
    "PaymentMethodSchemaResponse",
    "PaymentMethodType",
    "PaymentMethodsRequest",
    "PaymentMethodsResponse",
    "PaymentRailMarkup",
    "PaymentRailMarkupType",
    "PaymentRailRequest",
    "PaymentRailResponse",
    "PhoneNumber",
    "ProcessInvoiceRequest",
    "ProfileRequest",
    "ProfileResponse",
    "RepresentativeId",
    "RepresentativeRequest",
    "RepresentativeResponse",
    "Responsibilities",
    "SSN",
    "TaxID",
    "TransactionId",
    "TransactionResponse",
    "TransactionResponseExpanded",
    "TransactionStatus",
    "UnauthorizedError",
    "bank_lookup",
    "commons",
    "counterparty",
    "entity",
    "entity_users",
    "invoice",
    "ocr",
    "organization",
    "payment_method",
    "payment_method_schema",
    "process_invoice",
    "representative",
    "transaction",
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from ...core.api_error import ApiError
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_headers import remove_none_from_headers
from ...environment import MercoaEnvironment
from .types.comment_id import CommentId
from .types.comment_request import CommentRequest
from .types.comment_response import CommentResponse
from .types.document_response import DocumentResponse
from .types.invoice_id import InvoiceId
from .types.invoice_request import InvoiceRequest
from .types.invoice_response import InvoiceResponse


class InvoiceClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    def create(self, *, request: InvoiceRequest) -> InvoiceResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "invoice"),
            json=jsonable_encoder(request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InvoiceResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, invoice_id: InvoiceId) -> InvoiceResponse:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InvoiceResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(self, invoice_id: InvoiceId, *, request: InvoiceRequest) -> InvoiceResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InvoiceResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(self, invoice_id: InvoiceId) -> None:
        _response = httpx.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_documents(self, invoice_id: InvoiceId) -> typing.List[DocumentResponse]:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/documents"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DocumentResponse], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_comments(self, invoice_id: InvoiceId) -> typing.List[CommentResponse]:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comments"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[CommentResponse], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def add_comment(self, invoice_id: InvoiceId, *, request: CommentRequest) -> CommentResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comment"),
            json=jsonable_encoder(request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CommentResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def edit_comment(self, invoice_id: InvoiceId, comment_id: CommentId, *, request: CommentRequest) -> CommentResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comment/{comment_id}"),
            json=jsonable_encoder(request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CommentResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete_comment(self, invoice_id: InvoiceId, comment_id: CommentId) -> None:
        _response = httpx.request(
            "DELETE",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comment/{comment_id}"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_vendor_link(self, invoice_id: InvoiceId) -> str:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/vendorlink"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(str, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def send_vendor_email(self, invoice_id: InvoiceId) -> None:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/sendVendorEmail"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncInvoiceClient:
    def __init__(
        self, *, environment: MercoaEnvironment = MercoaEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    async def create(self, *, request: InvoiceRequest) -> InvoiceResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "invoice"),
                json=jsonable_encoder(request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InvoiceResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, invoice_id: InvoiceId) -> InvoiceResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InvoiceResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(self, invoice_id: InvoiceId, *, request: InvoiceRequest) -> InvoiceResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}"),
                json=jsonable_encoder(request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(InvoiceResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(self, invoice_id: InvoiceId) -> None:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "DELETE",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_documents(self, invoice_id: InvoiceId) -> typing.List[DocumentResponse]:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/documents"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[DocumentResponse], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_comments(self, invoice_id: InvoiceId) -> typing.List[CommentResponse]:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comments"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[CommentResponse], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def add_comment(self, invoice_id: InvoiceId, *, request: CommentRequest) -> CommentResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comment"),
                json=jsonable_encoder(request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CommentResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def edit_comment(
        self, invoice_id: InvoiceId, comment_id: CommentId, *, request: CommentRequest
    ) -> CommentResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comment/{comment_id}"),
                json=jsonable_encoder(request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(CommentResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete_comment(self, invoice_id: InvoiceId, comment_id: CommentId) -> None:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "DELETE",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/comment/{comment_id}"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_vendor_link(self, invoice_id: InvoiceId) -> str:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/vendorlink"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(str, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def send_vendor_email(self, invoice_id: InvoiceId) -> None:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", f"invoice/{invoice_id}/sendVendorEmail"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
            )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AccountType,
    BusinessProfileRequest,
    BusinessProfileResponse,
    BusinessType,
    Ein,
    EntityAddPayeesRequest,
    EntityId,
    EntityRequest,
    EntityResponse,
    EntityStatus,
    EntityUpdateRequest,
    IndividualProfileRequest,
    IndividualProfileResponse,
    InvoiceMetricsResponse,
    ProfileRequest,
    ProfileResponse,
    TaxID,
)

__all__ = [
    "AccountType",
    "BusinessProfileRequest",
    "BusinessProfileResponse",
    "BusinessType",
    "Ein",
    "EntityAddPayeesRequest",
    "EntityId",
    "EntityRequest",
    "EntityResponse",
    "EntityStatus",
    "EntityUpdateRequest",
    "IndividualProfileRequest",
    "IndividualProfileResponse",
    "InvoiceMetricsResponse",
    "ProfileRequest",
    "ProfileResponse",
    "TaxID",
]



# This file was auto-generated by Fern from our API Definition.

from .attachments import Attachments
from .email_ocr_request import EmailOcrRequest
from .ocr_mailbox import OcrMailbox
from .ocr_response import OCRResponse

__all__ = ["Attachments", "EmailOcrRequest", "OCRResponse", "OcrMailbox"]
